﻿import os
from datetime import datetime
import typing

import win32com.client

from . import active_session
from . import options


def _check_for_open_sessions():
    try:
        sap_gui = win32com.client.GetObject("SAPGUI")
        application = sap_gui.GetScriptingEngine
        connection = application.Children(0)
        active_session.session = connection.Children(0)
        return True
    except Exception as ex:
        print(ex)
        return False


def _get_session(connection: win32com.client.CDispatch) -> typing.NoReturn:
    _message = 'It was not possible to get the session'
    session_alive = False
    timer_init = datetime.now().timestamp()

    while not session_alive:
        try:
            if datetime.now().timestamp() - timer_init > 30:
                print(_message)
                exit(1)
            active_session.session = connection.Children(0)
            session_alive = True
        except Exception:
            print(_message)
            exit(1)
    if type(active_session.session) != win32com.client.CDispatch:
        print(_message)
        exit(1)

    print("Session startet")


def _get_connection(application: win32com.client.CDispatch) -> win32com.client.CDispatch:
    app_alive = False
    timer_init = datetime.now().timestamp()
    connection = None

    while not app_alive:
        try:
            if datetime.now().timestamp() - timer_init > 30:
                print('It was not possible to get the connection')
                exit(1)
            connection = application.Children(0)
            app_alive = True
        except Exception as ex:
            print(f"{ex}")
            app_alive = False

    if type(connection) != win32com.client.CDispatch:
        print('It was not possible to get the connection')
        exit(1)

    return connection


def _get_application(sap_gui_auto: win32com.client.CDispatch) -> win32com.client.CDispatch:
    if type(sap_gui_auto) != win32com.client.CDispatch:
        print('It was not possible to get the application')
        exit(1)

    application = sap_gui_auto.GetScriptingEngine
    if type(application) != win32com.client.CDispatch:
        print('It was not possible to get the application')
        exit(1)

    return application


def _wait_for_sap_gui() -> win32com.client.CDispatch:
    is_open = False
    timer_init = datetime.now().timestamp()
    while not is_open:  # Es wird auf den Start vom SAP-Client bis zu 30 Sekunden gewartet
        try:
            if datetime.now().timestamp() - timer_init > 30:
                print('Cannot start SAP GUI')
            return win32com.client.GetObject("SAPGUI")
        except Exception as ex:
            print(f"SAP Gui still not found! ({ex})")
            is_open = False


class Session:
    _system: options().sap_systems
    _user: str = ''
    _pwd: str = ''
    _client: str = "10"
    _language: str = "DE"

    @classmethod
    def set_session_data(cls, system_name: options().sap_systems = None, user_name: str = None, password: str = None,
                         client: str = "10",
                         language: str = "DE") -> typing.NoReturn:
        if system_name is not None:
            cls._system = system_name
        if user_name is not None:
            cls._user = user_name
        if password is not None:
            cls._pwd = password
        if client != "10":
            cls._client = client
        if language != "DE":
            cls._language = language

    @staticmethod
    def destroy() -> typing.NoReturn:
        active_session.session = None

    def error(self):
        # Close the sap session and exit the application
        raise NotImplementedError

    @classmethod
    def _open_sap_gui(cls):
        os.system(
            f'start sapshcut.exe -system="{cls._system.value}" -sysname="{cls._system.value}" -language={cls._language}'
            f' -client={cls._client} -user={cls._user} -password={cls._pwd} -maxgui -type="Transaction" '
            f'-command="SESSION_MANAGER" -title="Automation"')

    def create(self):
        """
            SAP wird über sapshcut mit den SAP Credentials gestartet. Bei SSO wird IMMER der Login vom SSO-User genutzt,
            abweichende Login-Daten sind nicht möglich. Da müsste man so wie bei AA ein Login-Script bauen.
            Sollte bereits eine SAP Session existieren dann wird sich dort draufgehangen 
            und keine neue SAP-Instanz geöffnet.
    
            Parameter:
    
            Rückgabewerte:
                - Erfolgreich = True + Log in einer Liste
                - nicht Erfolgreich = False + Log in einer Liste
        """

        try:
            self._create()
        except Exception as ex:
            print(f'Unbekannter Fehler beim Start von SAP ({ex})')
            exit(1)

    def _create(self):
        if _check_for_open_sessions():  # Existiert bereits eine Session wird sich mit der verbunden
            print("SAP Session existiert bereits, verbunden")
        if self._preconditions() is False:
            print('Preconditions where not met!')
            exit(1)

        self._open_sap_gui()

        sap_gui_auto = _wait_for_sap_gui()

        # Ab hier folgt grundsätzlich eine 1:1 Übersetzung aus den üblichen VBS Scripten
        # mit zusätzlichen Fehlerabfragen und Delays

        application = _get_application(sap_gui_auto)

        connection = _get_connection(application)

        _get_session(connection)

    @classmethod
    def _preconditions(cls):
        if cls._system is None:
            print("You need to specify a sap system to connect with.")
            exit(1)

        if cls._user is None:
            print('You need to provide a user name.')
            exit(1)

        if cls._pwd is None:
            print('No password provided.')
            exit(1)

        if cls._client == "":
            cls._client = "010"

        if cls._language == "":
            cls._language = "DE"

        return True


if __name__ == '__main__':
    raise NotImplementedError
